# -*- coding: utf-8 -*-

import sys
import numpy as np
import os
import getopt
import webbrowser

def usage():
    print ("\nUsage:")
    print (sys.argv[0] + " -s [sampling period] -m [mode] -i [input file] -e [simulation end time]\n")

def trick():
    new = 2
    url = "http://youtu.be/QtApW_wes3Q?t=16s"
    webbrowser.open(url,new=new)

class Module:
    def __init__(self, time, byte, sampling, mode):
        self.time      = time
        self.byte      = byte
        assert (len(time) == len(byte))
        self.sampling  = sampling
        self.startTime = float(int(time[0]))
        self.mode = mode

    def IterateUntilSampleReached (self, callback):
        values = []
        for i in range(0, len(self.time)):
            while (self.time[i] > self.startTime + self.sampling):
                callback (values)
                values = []
                self.startTime += self.sampling

            if self.startTime <= self.time[i] and self.time[i] <= self.startTime + self.sampling:
                    values.append (self.byte[i])
        if self.mode != "queueStat":
            callback (values)
        else:
            callback(values)
            callback(["x"])

class Throughput(Module):
    def __init__(self, time, byte, sampling):
        Module.__init__(self, time, byte, sampling, "throughput")
        self.oldMax = 0

    def CalcCallback (self, values):
        try:
            th = values[-1]
            tmp = th
            th = th - self.oldMax
            #check necessario dopo l'introduzione del 4° parametro, altrimenti il 1° valore nuovo è sempre < 0
            if th < 0:
                th = 0.0
            self.oldMax = tmp
        except:
            th = 0.0

        print (str(self.startTime) + " " + str(th))

    def GetOutput (self):
        oldmax = 0
        self.IterateUntilSampleReached (self.CalcCallback)


class Queue(Module):
    def __init__(self, time, byte, sampling):
        Module.__init__(self, time, byte, sampling, "queue")
        self.oldSample = 0
        print ("#TIME MAX MEAN MIN")

    def CalcCallback (self, values):
        if values == []:
            print (str(self.startTime) + " " + str(self.oldSample) + " " + str(self.oldSample) + " " +
                    str(self.oldSample))
        else:
            print (str(self.startTime) + " " + str(int(max(values))) + " " +
                str(int(np.mean(values))) + " " + str(int(min(values))))
            self.oldSample = int(values[-1])

    def GetOutput (self):
        self.IterateUntilSampleReached (self.CalcCallback)

class QueueStat(Queue):

    def __init__(self, time, byte, sampling):
        Module.__init__(self, time, byte, sampling, "queueStat")
        self.oldSample = 0
        self.maxTotal = []
        self.meanTotal = []
        self.minTotal = []
        print ("#MAX MEAN MIN")

    def CalcCallback (self, values):
        if values == []:
            self.maxTotal.append(self.oldSample)
            self.meanTotal.append(self.oldSample)
            self.minTotal.append(self.oldSample)
        elif values[0] != "x":
            self.maxTotal.append(int(max(values)))
            self.meanTotal.append(int(np.mean(values)))
            self.minTotal.append(int(min(values)))

            self.oldSample = int(values[-1])
        else:
            print ( str(max(self.maxTotal)) + " " + str(int(np.mean(self.meanTotal))) + " " + str(min(self.minTotal)) )


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hm:i:s:se:", ["help", "mode",
            "input", "sampling", "simend"])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(2)

    mode      = None
    inputFile = None
    sampling  = float(0.1)
    simend    = None

    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-i", "--input"):
            inputFile = a
        elif o in ("-s", "--sampling"):
            try:
                sampling = float(a)
                assert (sampling > 0), trick()
            except ValueError, e:
                print ("sampling should be a float, not " + a)
        elif o in ("-m", "--mode"):
            mode = a
        elif o in ("-e", "--simend"):
            try:
                simend = float(a)
                assert (simend > sampling), trick()
            except ValueError, e:
                print ("simulation end should be a float, not " + a)
        else:
            assert False, "unhandled option"

    if mode is None or inputFile is None:
        usage ()
        sys.exit (2)
    if mode == "throughput" and simend is None:
        print "\nWith \'throughput\' mode the 4th parameter is mandatory."
        usage()
        sys.exit(2)

    if os.path.isfile(inputFile):
        inputFd = open(inputFile, "r")
    else:
        print "\nAre you Stronzo Bestiale? " + inputFile + " does not exists.\n"
        sys.exit(2)

    time = []
    byte = []

    for line in inputFd:
        if line[0] == "#":
            pass
        else:
            column = line.split()
            time.append(float(column[0]))
            byte.append(float(column[1]))
    while time[-1] < simend:
        time.append(time[-1] + 1.0)
        byte.append(0.0)

    x = None
    if mode == "throughput":
        x = Throughput(time,byte,sampling)
    elif mode == "queue":
        x = Queue(time,byte,sampling)
    elif mode == "queueStat":
        x = QueueStat(time,byte,sampling)
    else:
        print ("\nmode = throughput | queue | queueStat. You inserted \'" + mode + "\'\n")
        sys.exit(2)

    x.GetOutput ()

if __name__ == "__main__":
    main()


